__author__ = "Alon B.R."

import pygame

from pygame import Vector2, Surface

pygame.init()


class _Particle2D:
    def __init__(
        self,
        x: int,
        y: int,
        vel_x: float | int,
        vel_y: float | int,
        accel_x: float | int,
        accel_y: float | int,
        time: float | int,
    ) -> None:

        self.pos: Vector2 = Vector2(x, y)
        self.vel: Vector2 = Vector2(vel_x, vel_y)
        self.accel: Vector2 = Vector2(accel_x, accel_y)

        self.time = time

    def update(
        self,
        deltaTime: float | int = 1,
        deltaTimeMultiply: float | int = 1,
    ) -> None:
        self.vel += self.accel
        self.vel.x *= deltaTime * deltaTimeMultiply
        self.vel.y *= deltaTime * deltaTimeMultiply

        self.pos.x += self.vel.x * deltaTime * deltaTimeMultiply
        self.pos.y += self.vel.y * deltaTime * deltaTimeMultiply

    def decay(
        self,
        decayAmount: float | int,
        deltaTime: float | int = 1,
        deltaTimeMultiply: float | int = 1,
    ) -> None:

        self.time -= decayAmount * deltaTime * deltaTimeMultiply

    @property
    def timeout(self) -> bool:
        return self.time <= 0


class ImageParticle2D(_Particle2D):
    def __init__(
        self,
        x: int,
        y: int,
        img: Surface,
        vel_x: float | int,
        vel_y: float | int,
        accel_x: float | int,
        accel_y: float | int,
        time: float | int,
    ) -> None:
        super().__init__(x, y, vel_x, vel_y, accel_x, accel_y, time)
        self.img: Surface = img

    def render(self, screen: Surface) -> None:
        screen.blit(self.img, self.pos)


class RectParticle2D(_Particle2D):
    def __init__(
        self,
        x: int,
        y: int,
        color: tuple,
        vel_x: float | int,
        vel_y: float | int,
        accel_x: float | int,
        accel_y: float | int,
        time: float | int,
    ) -> None:
        super().__init__(x, y, vel_x, vel_y, accel_x, accel_y, time)
        self.color: tuple = color

    def render(self, screen: pygame.Surface, width: int = 0) -> None:
        pygame.draw.rect(
            screen, self.color, (self.pos.x, self.pos.y, self.time, self.time), width
        )


class CircleParticle2D(_Particle2D):
    def __init__(
        self,
        x: int,
        y: int,
        color: tuple,
        vel_x: float | int,
        vel_y: float | int,
        accel_x: float | int,
        accel_y: float | int,
        time: float | int,
    ) -> None:
        super().__init__(x, y, vel_x, vel_y, accel_x, accel_y, time)
        self.color: tuple = color

    def render(self, screen: pygame.Surface, width: int = 0) -> None:
        pygame.draw.circle(
            screen, self.color, self.pos, self.time, width, draw_top_left=True
        )



__author__ = "Alon B.R."
import pygame
from pygame import Rect

pygame.init()

# from PGB import buttons, hitboxes, particles, tilebased, utils


class HitBox2D:
    """
    Better than pygame.Rect because of floating point accuracy
    and this class has more functions and everybody knows that more functions > less functions.
    """

    def __init__(
        self, x: float | int, y: float | int, width: float | int, height: float | int
    ) -> None:
        self.x: float | int = x
        self.y: float | int = y
        self.width: float | int = width
        self.height: float | int = height

    def collideHitBox(self, other) -> bool:
        return (
            (self.left >= other.left and self.left <= other.right)
            or (self.right >= other.left and self.right <= other.right)
        ) and (
            (self.top >= other.top and self.top <= other.bottom)
            or (self.bottom >= other.top or self.bottom <= other.bottom)
        )

    def collideLikeHitBox(
        self,
        likeOther: tuple[float | int, float | int, float | int, float | int]
        | list[float | int, float | int, float | int, float | int],
    ) -> bool:

        oleft = likeOther[0]
        oright = likeOther[1]
        otop = likeOther[2]
        obottom = likeOther[3]

        return (
            (self.left >= oleft and self.left <= oright)
            or (self.right >= oleft and self.right <= oright)
        ) and (
            (self.top >= otop and self.top <= obottom)
            or (self.bottom >= otop or self.bottom <= obottom)
        )

    def collidePoint(self, point: tuple[float | int, float | int]) -> bool:
        return (point[0] > self.x and point[0] < (self.x + self.width)) and (
            point[1] > self.y and point[1] < (self.y + self.height)
        )

    def render(self, screen: pygame.Surface) -> None:
        pygame.draw.rect(
            screen, (255, 0, 0), self.get_like_rect, 3
        )  # TODO add more options

    def addHitBox(self, other) -> None:
        self.x += other.x
        self.y += other.y

    def subHitBox(self, other) -> None:
        self.x -= other.x
        self.y -= other.y

    def mulHitBox(self, other) -> None:
        self.x *= other.x
        self.y *= other.y

    def divHitBox(self, other) -> None:
        self.x /= other.x
        self.y /= other.y

    def modHitBox(self, other) -> None:
        self.x %= other.x
        self.y %= other.y

    @property
    def get_rect(self) -> Rect:
        return Rect(self.x, self.y, self.width, self.height)

    @property
    def get_like_rect(
        self,
    ) -> tuple[float | int, float | int, float | int, float | int]:
        return (self.x, self.y, self.width, self.height)

    @property
    def get_pos(self) -> tuple[float | int, float | int]:
        return (self.x, self.y)

    @property
    def bottom(self) -> float | int:
        return self.y + self.height

    @property
    def top(self) -> float | int:  # idk why
        return self.y

    @property
    def right(self) -> float | int:
        return self.x + self.width

    @property
    def left(self) -> float | int:  # idk why
        return self.x


print("Welcome to the Pygame Builtins libarary")



from setuptools import find_packages, setup

setup(
    name="PygameBuiltins",
    packages=find_packages(include=["PygameBuiltins"]),
    version="0.1.0",
    description="A pygame wrapper with usefull classes and functions.",
    author="MrCatNerd#0669",
    license="MIT",
    install_requires=["pygame"],
    setup_requires=["pytest-runner"],
    tests_require=["pytest==4.4.1"],
    test_suite="tests",
)



__author__ = "Alon B.R."

import pygame

from pygame import Surface

from PygameBuiltins import HitBox2D
from PygameBuiltins import HitBox2D

pygame.init()


class ImageButton2D:
    def __init__(
        self,
        x: float | int,
        y: float | int,
        img: Surface,
        working: bool = True,
    ) -> None:
        self.img: Surface = img
        self.hitbox: HitBox2D = HitBox2D(
            x, y, self.img.get_width(), self.img.get_height()
        )

        self.state: bool = False
        self.working: bool = working

    def update(self, mouseX: float | int, mouseY: float | int, click: bool) -> None:
        if self.hitbox.collidePoint((mouseX, mouseY)) and click and self.working:
            self.activate()
        elif self.state:
            self.reset()

    def reset(self) -> None:
        if not self.working:
            return
        self.state = False

    def activate(self) -> None:
        if not self.working:
            return
        self.state = True

    def switch(self) -> None:
        if not self.working:
            return
        self.state = not self.state

    def disable_button(self) -> None:
        self.working = False

    def enable_button(self) -> None:
        self.working = True

    def render(self, screen: Surface) -> None:
        screen.blit(self.img, self.hitbox.get_pos)


class ImageSwitch2D(ImageButton2D):
    def __init__(
        self,
        x: float | int,
        y: float | int,
        imgOff: Surface,
        imgOn: Surface,
        state: bool = True,
        working: bool = True,
    ) -> None:
        self.imgOff: Surface = imgOff
        self.imgOn: Surface = imgOn
        self.hitbox: HitBox2D = HitBox2D(
            x, y, self.img.get_width(), self.img.get_height()
        )

        self.state: bool = False
        self.working: bool = working

        self.state = state

    def update(self, mouseX: float | int, mouseY: float | int, click: bool) -> None:
        if self.hitbox.collidePoint((mouseX, mouseY)) and click and self.working:
            self.switch()

    def render(self, screen: Surface) -> None:
        screen.blit(
            {False: self.imgOff, True: self.imgOn}[self.state], self.hitbox.get_pos
        )



__author__ = "Alon B.R."

import pygame
import math

from os.path import join as join_path

pygame.init()


def get_directions_from_degree(degree: float | int) -> tuple[float, float]:

    x_move = math.sin(degree)
    y_move = math.cos(degree)

    return (x_move, y_move)


def load_image(
    *paths: str, size: tuple | list | None = None, convert_alpha: bool = False
) -> pygame.Surface:

    srf: pygame.Surface = pygame.image.load(join_path(*paths))

    if size is not None:
        srf = pygame.transform.scale(srf, size)

    if convert_alpha:
        srf = srf.convert_alpha()
    else:
        srf = srf.convert()

    return srf
